#!/bin/sh
PASS="[1m[32m + [0m[1m"
FAIL="[1m[31m - [0m[1m"

startlog() {
    LOG=$1
    LOG_FIFO=$1.fifo
    mkfifo -m 0600   $LOG_FIFO 2>/dev/null || :
    touch $LOG
    chmod 0600 $LOG
    tee $LOG        <$LOG_FIFO &
    exec 3>&1 4>&2 1>$LOG_FIFO 2>&1
}

stoplog() {
    exec 1>&3- 2>&4-
    rm $LOG_FIFO
    unset LOG_FIFO
}

emergency() {
    echo $1
    echo -e "\a... press any key in 5 seconds to open shell ..."
    read -t 5 -n 1 -s || return 1
    if [[ -n "$LOG_FIFO" ]]; then
        bash --norc >&3 2>&4
    else
        bash --norc
    fi
    return
}

run() {
    eval "ERROR=''; trap 'eval \"ERROR=\$?\"' ERR; shopt -s -o verbose
$(</proc/self/fd/0)
shopt -u -o verbose" &>/tmp/run.$$.$1
    if [[ -z $ERROR ]]; then
        echo "$PASS$1[0m"
    else
        RESCUE=1
        echo "$FAIL$1[0m"
        cat /tmp/run.$$.$1 | egrep -v '^eval|^shopt' | sed "s/\(.*ERROR.*\)/\a[36m\1[0m/g"
    fi
}

parallel() {
    PATH=$PATH bash -c "ERROR=''; trap 'eval \"ERROR=\$?\"' ERR; shopt -s -o verbose
$(</proc/self/fd/0)
shopt -u -o verbose
exit \$ERROR" &>/tmp/run.$$.$1 &
    eval "PID_$1=$!"
    NEED="$NEED $1"
}

need() {
    if wait $(eval "echo \$PID_$1") &>/dev/null; then
        echo "$PASS$1[0m"
    else
        [[ "$?" == "127" ]] && return
        RESCUE=1
        echo "$FAIL$1[0m"
        cat /tmp/run.$$.$1 | egrep -v '^eval|^shopt' | sed "s/\(.*ERROR.*\)/\a[36m\1[0m/g"
    fi
}

needall() {
    for i in $NEED; do need $i; done
    rm /tmp/run.$$.*
    if [[ "$RESCUE" == "1" ]]; then
        if [[ "$0" == "/etc/runit/1" ]]; then
            if emergency 'Some errors happens while BOOT (stage 1)!'; then
                echo '... press any key in 3 seconds if you wanna shutdown now ...'
                read -t 3 -n 1 -s && exit 100
            fi
        elif [[ "$0" == "/etc/runit/3" ]]; then
            emergency 'Some errors happens while SHUTDOWN (stage 3)!'
        fi
    fi
}

